"""
Test Poi CSV export.
"""

import io
import csv

import unittest

from zope import interface

from plone.testing.z2 import Browser
from plone.app import testing as pa_testing

from retrans.poi import interfaces
from retrans.poi import testing


class TestPoiCSVExport(unittest.TestCase):
    """
    Test Poi CSV export.
    """

    layer = testing.RETRANS_POI_ISSUE_INTEGRATION_TESTING

    def setUpBrowser(self):
        """
        Start a logged in browser session.
        """
        app = self.layer['app']
        self.browser = Browser(app)
        self.browser.handleErrors = False  # Don't get HTTP 500 pages

        portal = self.layer['portal']
        portal_url = portal.absolute_url()

        self.browser.open(portal_url + '/login_form')
        self.browser.getControl(
            name='__ac_name').value = pa_testing.SITE_OWNER_NAME
        self.browser.getControl(
            name='__ac_password').value = pa_testing.SITE_OWNER_PASSWORD
        self.browser.getControl(name='submit').click()
        self.assertIn(
            "You are now logged in", self.browser.contents.decode('utf-8'),
            'Missing login confirmation message')

    def test_empty_content(self):
        """
        The empty tracker and issue have enough set up to work for tests.
        """
        tracker = self.layer['tracker']
        self.setUpBrowser()
        self.browser.open(tracker.absolute_url())
        self.browser.open(tracker['1'].absolute_url())
        self.assertIn(
            'complexity', self.browser.contents, 'Missing custom fields')

    def test_export_empty_issue(self):
        """
        An empty issue can be exported to CSV.
        """
        tracker = self.layer['tracker']
        interface.alsoProvides(tracker.REQUEST, interfaces.IRetransPoiLayer)

        exporter = tracker.unrestrictedTraverse('@@poi-issue-export')
        raw_csv = exporter()
        reader = csv.DictReader(io.BytesIO(raw_csv))
        rows = list(reader)

        self.assertEquals(
            len(rows), 1,
            'Wrong number of exported CSV rows')
        row = rows[0]
        self.assertIn(
            'Complexity', row,
            'Exported CSV missing custom issue field')
        self.assertEqual(
            row['Complexity'], '',
            'Exported CSV wrong custom issue field value')

        self.assertNotIn(
            'Software Engineer Time', row,
            'Exported CSV includes ambiguous custom issue field label')
        self.assertIn(
            'Software Engineer - Estimated', row,
            'Exported CSV missing custom issue field')
        self.assertIn(
            'System Engineer - Actual', row,
            'Exported CSV missing custom issue field')
